#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include "pieces_moves.h"

// Initializare LCD la dimensiunea 2004
LiquidCrystal_I2C lcd(0x27, 20, 4);  

// Butonul de START
#define START 6
// Pinii Joystick-ului din stanga
#define VRx_left A0
#define VRy_left A1
#define SW_left 6
// Pinii Joystick-ului din stanga
#define VRx_right A2
#define VRy_right A3
#define SW_right 4
// Dimensiunile LCD-ului (20 coloane, 4 randuri)
#define LCD_COLS 20
#define LCD_ROWS 4

int score1 = 0, score2 = 0;
boolean game = 0; 


void setup() {  
  // Initializare buton de  start
  pinMode(START, INPUT);

  // Initializare pini Joystick
  pinMode(VRx_left, INPUT);
  pinMode(VRy_left, INPUT);
  pinMode(SW_left, INPUT_PULLUP);
  pinMode(VRx_right, INPUT);
  pinMode(VRy_right, INPUT);
  pinMode(SW_right, INPUT_PULLUP);

  // Initializare LCD
  lcd.init();
  lcd.backlight();
  lcd.clear();

  // Afisare mesaj de inceput
  lcd.setCursor(5, 1);
  lcd.print("Pong  Game"); 
  delay(2000);
}

void loop() {
  // Jocul nu a inceput inca
  while (game == 0) {
    initGame();

    // Verifica starea butonului de start 
    if (digitalRead(START) == HIGH) {
      game = 1;   // Incepe jocul
      delay(500);
    }
  }

  // Citeste informatii de la Joystick
  readCommand();

  // Testeaza finalul jocului
  int endgame = checkEndgame();

  if (endgame == 0) {
    movePaddles();
    moveBall();
    setPieces();
    printGame();
  }

  else
    printScore();
}


// Initializeaza pozitiile elementelor si directia de joc
void initGame() {
  for (int i = 0; i < CHARS_PER_COL; i++) {
    // Activare pozitii initiale ale cursorului
    if (12 <= i && i <= 19) {
    paddleLeft[i] = LPCHAR;
    paddleRight[i] = RPCHAR;
  }
  else {
    paddleLeft[i] = 0;
    paddleRight[i] = 0;
  }

  // Activare pozitie initiala a mingii
  if (i == 15)
    ball_vect[i] = BCHAR;
  else
      ball_vect[i] = 0;
  }

  // Mingea se afla initial pe coloana din centru
  ball_column = LCD_COLS / 2;

  // Directia opusa fara de cum s-a terminat runda trecuta
  if (direction == 'L')
    direction = 'R';
  else 
    direction = 'L';

  // Afisare mesaj "Press button to START."
  lcd.clear();
  lcd.setCursor(4, 1);
  lcd.print("Press button");
  lcd.setCursor(6, 2);
  lcd.print("to START");
  delay(200);
}

// Determina daca cursoarele trebuie mutate
void readCommand() {
  // Citire informatii de la Joystick
  int paddleL_xPos = analogRead(VRx_left);
  int paddleR_xPos = analogRead(VRx_right);

  paddleL_up = 0; paddleL_down = 0; 
  paddleR_up = 0; paddleR_down = 0;

  // Mapare pozitie Joystick
  int mapL = map(paddleL_xPos, 0, 1023, 0, 10);
  int mapR = map(paddleR_xPos, 0, 1023, 0, 10);

  if (mapL < 4)
    paddleL_up = 1;
  else if (mapL > 6)
    paddleL_down = 1;

  if (mapR < 4)
    paddleR_up = 1;
  else if (mapR > 6)
    paddleR_down = 1;
}

// Seteaza vectorii care descriu 
void setPieces() {
  // Setare paddle-ul din stanga
  for (int i = 0; i < CHARS_PER_COL; i++) {
    if (paddleLeft[i] == LPCHAR) {
      paddleL_row = (int) i / 8;
      break;
    }
  }
  for (int i = 0; i < CHARS_PER_BLOCK; i++) {
    paddleLeft_1[i] = paddleLeft[paddleL_row * 8 + i];
    paddleLeft_2[i] = paddleLeft[(paddleL_row + 1) * 8 + i];
  }

  // Setare paddle-ul din dreapta
  for (int i = 0; i < CHARS_PER_COL; i++) {
    if (paddleRight[i] == RPCHAR) {
      paddleR_row = (int) i / 8;
      break;
    }
  }
  for (int i = 0; i < CHARS_PER_BLOCK; i++) {
    paddleRight_1[i] = paddleRight[paddleR_row * 8 + i];
    paddleRight_2[i] = paddleRight[(paddleR_row + 1) * 8 + i];
  }

  // Setarea mingii
  for (int j = 0; j < CHARS_PER_BLOCK; j++) {
    ball[j] = 0;
  }
  for (int i = 0; i < CHARS_PER_COL; i++) {
    if (ball_vect[i] == 1) {
      ball[i % 8] = 1;
      ball_row = (int) i / 8;
    }
  }
}

// Testeaza coliziunea dintre un paddle si minge
int checkCollision(byte paddle[CHARS_PER_COL]) {
  int ball_pos = 0, paddle_pos = 0;

  for (int i = 0; i < CHARS_PER_COL; i++) {
    if (ball_vect[i] != 0) {
      ball_pos = ball_vect[i];
      paddle_pos = paddle[i];
    }
  }

  // Daca positiile sunt la fel avem coliziune
  if (paddle_pos != 0)
    return 1;
  else 
    return 0;
}

// Testeaza sfarsitul jocului
int checkEndgame() {
  // Verificare coliziune dintre paleta stanga si minge
  if (ball_column == 0) {
    int res = checkCollision(paddleLeft);

    // Daca paleta a lovit mingea, schimba directia la dreapta
    if (res == 1) {
      direction = 'R';
      return 0;
    }
    // S-a inscris un punct de jucatorul oponent
    else {
      game = 0;
      score2 = score2 + 1;
      return 1;
    }
  }

  // Verificare coliziune dintre paleta dreapta si minge
  if (ball_column == LCD_COLS - 1) {
    int res = checkCollision(paddleRight);

    // Daca paleta a lovit mingea, schimba directia la stanga
    if (res == 1) {
      direction = 'L';
      return 0;
    } 
    // S-a inscris un punct de jucatorul oponent
    else {
      game = 0;
      score1 = score1 + 1;
      return 1;
    }
  }

  return 0;
}

// Afiseaza pe LCD jocul
void printGame() {  
  // Creare caractere custom  
  lcd.createChar(0, paddleLeft_1);
  lcd.createChar(1, paddleLeft_2);
  lcd.createChar(2, paddleRight_1);
  lcd.createChar(3, paddleRight_2);
  lcd.createChar(4, ball);

  lcd.clear();

  // Scriere caractere custom pentru cursor stanga
  lcd.setCursor(0, paddleL_row);
  lcd.write(byte(0));
  lcd.setCursor(0, paddleL_row + 1);
  lcd.write(byte(1));

  // Scriere caractere custom pentru cursor dreapta
  lcd.setCursor(LCD_COLS - 1, paddleR_row);
  lcd.write(byte(2));
  lcd.setCursor(LCD_COLS - 1, paddleR_row + 1);
  lcd.write(byte(3));

  // Scriere caractere custom pentru minge
  lcd.setCursor(ball_column, ball_row);
  lcd.write(byte(4));

  // Scriere scor sub forma "1 - 1"
  lcd.setCursor(8, 0);
  lcd.print(score1);
  lcd.setCursor(10, 0);
  lcd.print("-");
  lcd.setCursor(12, 0);
  lcd.print(score2);
}

// Afiseaza pe LCD scorul cand se castiga un joc
void printScore() {
  // Afisare mesaj "Player_x won!"
  lcd.clear();
  lcd.setCursor(3, 1);
  if (direction == 'L')
    lcd.print("Player 2 won!");
  else
    lcd.print("Player 1 won!");
  delay(1000);

  // Afisare scor
  lcd.clear();
  lcd.setCursor(7, 0);
  lcd.print("Score");

  lcd.setCursor(1, 2);
  lcd.print("Player1");
  lcd.setCursor(4, 3);
  lcd.print(score1);

  lcd.setCursor(12, 2);
  lcd.print("Player2");
  lcd.setCursor(15, 3);
  lcd.print(score2);

  delay(1000);
}
